/* vMac 0.1 Source
   By Philip Cummins (PC)
   Copyright PC 1997 */
   
/* PC - RTC (Real Time Clock) Functions */

#include "RTC.h"
#include "custom.h"

RTC_Ty RTC;

UBYTE RTC_Mode = 0;
UBYTE RTC_Cmd  = 0;
UBYTE RTC_Data = 0;
UBYTE RTC_Cnt  = 0;

int   RTC_Init (void)
{
  int Counter = 0;
  FILE *PRAM_File;
  int File_Size = 0;
  
  RTC.Seconds_0 = RTC.Seconds_1 = RTC.Seconds_2 = RTC.Seconds_3 = 0;
  RTC.Test = RTC.Write_Protect = 0;
  for (Counter = 0; Counter <= 23; Counter++) /* Docs only say 20, but there are 24 */
  {
    RTC.RAM[Counter] = 0;
  }
  PRAM_File = fopen("vMac.PRAM", "rb");
  if (PRAM_File == NULL)
  {
    printf("The vMac.PRAM could not be found, will create one.\n");
  }
  else
  {
    File_Size = fread(RTC.RAM, 24, 1, PRAM_File);
    if (File_Size != 1)
    {
      printf("The vMac.PRAM is not 24 Bytes large.\n");
      fclose(PRAM_File);
      return 1;
    }
    fclose(PRAM_File);
  }
  printf("vMac.PRAM loaded successfully.\n");
  return 0;
}

void  RTC_Reset (void)
{
  NULL;
}

void RTC_Interrupt (void)
{
  long Seconds = 0;
  
  Seconds = (RTC.Seconds_3 << 24) + (RTC.Seconds_2 << 16) + (RTC.Seconds_1 << 8) + RTC.Seconds_0;
  Seconds++;
  RTC.Seconds_0 = Seconds & 0xFF;
  RTC.Seconds_1 = (Seconds & 0xFF00) >> 8;
  RTC.Seconds_2 = (Seconds & 0xFF0000) >> 16;
  RTC.Seconds_3 = (Seconds & 0xFF000000) >> 24;
}

void  RTC_Put (UBYTE in)
{
  RTC_Data = (RTC_Data << 1) + in;
  RTC_Cnt++;
  if (RTC_Cnt == 8)
  {
    RTC_DoCmd();
  }
}

UBYTE RTC_Get  (void)
{
  return ((RTC_Data >> --RTC_Cnt) & 0x01);
}

void RTC_DoCmd (void)
{
  if (RTC_Mode == 0) /* This Byte is a RTC Command */
  {
    RTC_Cmd = RTC_Data;
    RTC_Data = 0;
    if ((RTC_Cmd & 0x80) == 0x00) /* Write Command */
    {
      RTC_Mode = 1; /* Set to switch to Write Mode */
      RTC_Cnt = 0;  /* Reset the Counter */
    }
    else /* Read Command */
    {
      RTC_Read(); /* Set up RTC for Reading Proper Data */
    }
  }
  else /* Data has come in that needs to be written */
  {
    RTC_Write();
  }
}

void RTC_Read (void)
{
  printf("RTC Read Reg %2x\n", RTC_Cmd);
  switch ((RTC_Cmd & 0x7C) >> 2)
  {
    case 0 :
      RTC_Data = RTC.Seconds_0; break;
    case 1 :
      RTC_Data = RTC.Seconds_1; break;
    case 2 :
      RTC_Data = RTC.Seconds_2; break;
    case 3 :
      RTC_Data = RTC.Seconds_3; break;
    case 4 : case 5 : case 6 : case 7 :
      RTC_Data = RTC.RAM[((RTC_Cmd & 0x0C) >> 2) + 0x20]; 
      break;
    case 8 : case 9 : case 10 : case 11 :
      RTC_Data = RTC.RAM[((RTC_Cmd & 0x0C) >> 2) + 0x10]; break;
    case 16 : case 17 : case 18 : case 19 :
    case 20 : case 21 : case 22 : case 23 :
    case 24 : case 25 : case 26 : case 27 :
    case 28 : case 29 : case 30 : case 31 :
      RTC_Data = RTC.RAM[((RTC_Cmd & 0x3C) >> 2)]; break;
    default :
      RTC_Data = 0;
      printf("RTC Read Reg Unknown, %2x\n", RTC_Cmd); break;
  }
  RTC_Cmd = RTC_Mode = 0;
}

void RTC_Write (void)
{
  printf("RTC Write Reg %2x %2x\n", RTC_Cmd, RTC_Data);
  switch ((RTC_Cmd & 0x7C) >> 2)
  {
    case 0 :
      RTC.Seconds_0 = RTC_Data; break;
    case 1 :
      RTC.Seconds_1 = RTC_Data; break;
    case 2 :
      RTC.Seconds_2 = RTC_Data; break;
    case 3 :
      RTC.Seconds_3 = RTC_Data; break;
    case 4 : case 5 : case 6 : case 7 :
      RTC.RAM[((RTC_Cmd & 0x0C) >> 2) + 0x20] = RTC_Data;
      break;
    case 8 : case 9 : case 10 : case 11 :
      RTC.RAM[((RTC_Cmd & 0x0C) >> 2) + 0x10] = RTC_Data; break;
    case 12 :
      RTC.Test = RTC_Data; break;
    case 13 :
      RTC.Write_Protect = RTC_Data; 
      RTC_Save(); break;
    case 16 : case 17 : case 18 : case 19 :
    case 20 : case 21 : case 22 : case 23 :
    case 24 : case 25 : case 26 : case 27 :
    case 28 : case 29 : case 30 : case 31 :
      RTC.RAM[((RTC_Cmd & 0x3C) >> 2)] = RTC_Data; break;
    default :
      RTC_Data = 0;
      printf("RTC Write Reg Unknown, %2x\n", RTC_Cmd); break;
  }
  RTC_Cmd = RTC_Data = RTC_Cnt = RTC_Mode = 0;
}

void RTC_Save (void)
{
  FILE *PRAM_File;
  int File_Size = 0;
  
  PRAM_File = fopen("vMac.PRAM", "wb");
  File_Size = fwrite(RTC.RAM, 24, 1, PRAM_File);
  if (File_Size != 1)
  {
    printf("vMac had problems writing the PRAM file\n");
  }
  fclose(PRAM_File);
}