/* vMac 0.1 Source
   By Philip Cummins (PC)
   Copyright PC 1997 */
   
/* PC - VIA Function Files */

#include <stdio.h>
#include "VIA.h"
#include "RTC.h"
#include "Keyboard.h"
#include "custom.h"

extern int InterruptNo;

VIA_Ty VIA;

UBYTE vOverlay = 1; /* Overlay Switch from Buffer A */
UBYTE vPage2 = 1;   /* Switch for Main and Alternate Screen Buffers */
UBYTE vSel = 0;     /* Switch for IWM operations */
UBYTE vVBL_Int = 0; /* Switch for Vertical Blanking Interrupts */
UBYTE vSec_Int = 0; /* Switch for 1 Second Interrupts */
UBYTE vKDR_Int = 0; /* Switch for Keyboard Data Ready Interrupts */
UBYTE vSR_Out = 0;  /* Switch to notify that Keyboard data ready */
UBYTE vDebug = 0;   /* Switch to turn VIA debug info on/off */

int  VIA_Init (void)
{
  VIA.ORA   = 0x7B;
  VIA.DDR_A = 0x7F;
  VIA.ORB   = 0x07;
  VIA.DDR_B = 0xB7;

  VIA.T1C_L = VIA.T1C_H = VIA.T1L_L = VIA.T1L_H = 0x00;
  VIA.T2_L  = VIA.T2_H  = VIA.SR    = VIA.ACR   = 0x00;
  VIA.PCR   = VIA.IFR   = VIA.IER   = VIA.ORA   = 0x00;
}

void  VIA_Reset (void)
{
  (void) VIA_Init();
}

UBYTE VIA_GetB (CPTR addr)
{  
  if ((addr & 1) == 0)
  {
    if (vDebug)
      {  printf("VIA GB %4lx\n", addr); }
    switch (addr >> 9)
	{
	  case kORB   :
	  
	    if ((VIA.PCR & 0xE0) == 0)
	    {
	      VIA.IFR &= 0xF7;
	    }
	  
	    if (Button())
	      { VIA.ORB |= 0x08; }
	    else
	      { VIA.ORB &= 0xF7; }
	    
	  	if (((VIA.ORB & 0x06) == 0x02) && ((VIA.DDR_B & 0x01) == 0x00))
        { VIA.ORB = ((VIA.ORB & 0xFE) + RTC_Get()); }
        
	    return VIA.ORB;   break;
	  case kDDR_B :
	    return VIA.DDR_B; break;
	  case kDDR_A :
	    return VIA.DDR_A; break;
	  case kT1C_L :
	    return VIA.T1C_L; break;
	  case kT1C_H :
	    return VIA.T1C_H; break;
	  case kT1L_L :
	    return VIA.T1L_L; break;
	  case kT1L_H :
	    return VIA.T1L_H; break;
	  case kT2_L  :
	    return VIA.T2_L;  break;
	  case kT2_H  :
	    return VIA.T2_H;  break;
	  case kSR    :
	    return VIA.SR;    break;
	  case kACR   :
	    return VIA.ACR;   break;
	  case kPCR   :
	    return VIA.PCR;   break;
	  case kIFR   :
	    return VIA.IFR;   break;
	  case kIER   :
	    return VIA.IER;   break;
	  case kORA   :
	  case kORA_H :
	    if ((VIA.PCR & 0xE) == 0)
	    {
	      VIA.IFR &= 0xFE;
	    }
	    return VIA.ORA;   break;
	}
  }
  else
  {
    return 0;
  }
}

UWORD VIA_GetW (CPTR addr)
{
  return 0;
}

ULONG VIA_GetL (CPTR addr)
{
  return 0;
}

void  VIA_PutB (CPTR addr, UBYTE in)
{  
  if ((addr & 1) == 0)
  {
    if (vDebug)
      { printf("VIA PB %4lx %2x\n", addr, in); }
    switch (addr >> 9)
	{
	  case kORB   :
	    
	    if ((VIA.PCR & 0xE0) == 0)
	    {
	      VIA.IFR &= 0xF7;
	    }
	    
	    if (((in & 0x06) == 0x02) && ((VIA.DDR_B & 0x01) == 0x01))
	    { RTC_Put(in & 0x01); }
	    
	    VIA.ORB   = in; break;
	  case kDDR_B :
	    VIA.DDR_B = in; break;
	  case kDDR_A :
	    VIA.DDR_A = in; break;
	  case kT1C_L :
	    VIA.T1C_L = in; break;
	  case kT1C_H :
	    VIA.T1C_H = in; break;
	  case kT1L_L :
	    VIA.T1L_L = in; break;
	  case kT1L_H :
	    VIA.T1L_H = in; break;
	  case kT2_L  :
	    VIA.T2_L  = in; break;
	  case kT2_H  :
	    VIA.T2_H  = in; break;
	  case kSR    :
	    VIA.SR    = in;
	    if (vSR_Out)
	      { Keyboard_Send(VIA.SR); }
	    break;
	  case kACR   :
	    VIA.ACR   = in;
	    switch ((VIA.ACR & 0x1C) >> 2)
	    {
	      case 0 : break; /* Do Nothing */
	      case 1 :
	      case 2 :
	      case 3 :
	        vSR_Out = 0;
	        VIA.SR = Keyboard_Get();
	        if (vKDR_Int)
	        {
	          VIA.IFR |= 0x04;
              specialflags |= SPCFLAG_DOINT;
              InterruptNo = 1;	          
	        }
	        break;
	      case 4 :
	      case 5 :
	      case 6 :
	      case 7 :
	        vSR_Out = 1;
	        break;
	    }
	    break;
	  case kPCR   :
	    VIA.PCR   = in; break;
	  case kIFR   :
	    VIA.IFR   = in; break;
	  case kIER   :
	    if ((in & 0x80) == 0)
	      { VIA.IER = VIA.IER & (0x7F - in); } /* Clear Enable Bits */
	    else
	      { VIA.IER = VIA.IER | (in & 0x7F); } /* Set Enable Bits */
	    
	    if ((VIA.IER & 0x04) == 0x04)
	      { vKDR_Int = 1; }
	    else
	      { vKDR_Int = 0; }
	      
	    if ((VIA.IER & 0x02) == 0x02)
	      { vVBL_Int = 1; }
	    else
	      { vVBL_Int = 0; }
	    
	    if ((VIA.IER & 0x01) == 0x01)
	      { vSec_Int = 1; }
	    else
	      { vSec_Int = 0; }
	    
	    break;
	  case kORA   :
	  case kORA_H :
	    
	    if ((VIA.PCR & 0xE) == 0)
	    {
	      VIA.IFR &= 0xFE;
	    }
        
        if ((in & 0x40) == 0x40)
	      { vPage2 = 1; }
	    else
	      { vPage2 = 0; }
	    
	    if ((in & 0x20) == 0x20)
	      { vSel = 1; }
	    else
	      { vSel = 0; }
	    
	    if ((in & 0x10) == 0x10)
	      { vOverlay = 1; }
	    else
	      { vOverlay = 0; }
	    
	    VIA.ORA   = in; break;
	}
  }
}

void  VIA_PutW (CPTR addr, UWORD in)
{
  NULL;
}

void  VIA_PutL (CPTR addr, ULONG in)
{
  NULL;
}